import click

@click.group()
def main():
    pass

@click.command('s3:push')
@click.option('-b', '--bucket', required=True)
@click.option('-p', '--public', default=False, is_flag=True)
@click.option('--acl')
@click.option('--prefix')
@click.option('--expires', type=int, default=86400)
@click.argument('ipa_file')
def s3(ipa_file, bucket, acl, prefix, public, expires):
    from .s3 import push

    url = push(
        ipa_path=ipa_file,
        bucket=bucket,
        acl=acl,
        prefix=prefix,
        public=public,
        expires=expires,
    )
    click.echo(url)

main.add_command(s3)



import attr
import boto3
import os.path
import typing

from .ipa import open_ipa
from .templates import DEFAULT_HTML, DEFAULT_MANIFEST, render_template

@attr.s(auto_attribs=True)
class S3:
    client: typing.Any
    bucket: str
    acl: str
    public: bool
    expires: int

    def put_object(self, path, data, content_type):
        self.client.put_object(
            Bucket=self.bucket,
            Key=path,
            Body=data,
            ACL=self.acl,
            ContentType=content_type,
        )

        url = self.client.generate_presigned_url(
            'get_object',
            Params=dict(
                Bucket=self.bucket,
                Key=path,
            ),
            ExpiresIn=self.expires,
        )
        if self.public:
            i = url.index('?')
            url = url[:i]
        return url

def push(
    ipa_path,
    bucket,
    *,
    acl=None,
    public=False,
    expires=86400,
    prefix=None,
):
    if not acl:
        acl = 'public-read' if public else 'private'

    if not prefix:
        prefix = os.path.splitext(os.path.basename(ipa_path))[0]

    s3 = S3(
        client=boto3.client('s3'),
        bucket=bucket,
        acl=acl,
        public=public,
        expires=expires,
    )

    image_url = None
    with open_ipa(ipa_path) as ipa:
        icon = ipa.find_best_icon()
        if icon is not None:
            with ipa.open_asset(icon.name) as fp:
                image_url = s3.put_object(f'{prefix}.png', fp, 'image/png')

    with open(ipa_path, 'rb') as fp:
        ipa_url = s3.put_object(f'{prefix}.ipa', fp, 'application/zip')

    manifest = render_template(DEFAULT_MANIFEST, context=dict(
        ipa=ipa,
        ipa_url=ipa_url,
        image_url=image_url,
    ))
    manifest_url = s3.put_object(
        f'{prefix}.plist',
        manifest.encode('utf8'),
        'text/xml',
    )

    html = render_template(DEFAULT_HTML, context=dict(
        ipa=ipa,
        image_url=image_url,
        manifest_url=manifest_url,
    ))
    url = s3.put_object(
        f'{prefix}.html',
        html.encode('utf8'),
        'text/html',
    )
    return url



import jinja2

DEFAULT_HTML = '''
<!doctype html>
<html>
<head>
    <title>{{ ipa.display_name }} v{{ ipa.short_version }} ({{ ipa.version }})</title>
    <meta name="viewport" content="initial-scale=1.0, user-scalable=no">
    <style type="text/css">
        body {
            font-family: Helvetica, Arial, sans-serif;
            text-align: center;
            color: #444;
            font-size: 18px;
        }
        img {
            display: block;
            margin: 1em auto;
            border: none;
            width: 120px;
            height: 120px;
            border-radius: 20px;
        }
        .btn, a.btn, a.btn:visited, a.btn:hover, a.btn:link {
            display: inline-block;
            border-radius: 3px;
            background-color: #0095c8;
            color: white;
            padding: .8em 1em;
            text-decoration: none;
        }
        a.btn:hover {
            background-color: #00bbfb;
            color: white;
        }
        p {
            color: #999
        }
    </style>
</head>
<body>
    <h1>{{ ipa.display_name }}</h1>
    <h2>{{ ipa.short_version }} ({{ ipa.version }})</h2>
    {% if image_url %}
    <a href="itms-services://?action=download-manifest&amp;url={{ manifest_url | urlencode }}">
        <img src="{{ image_url | safe }}">
    </a>
    {% endif %}
    <a
        href="itms-services://?action=download-manifest&amp;url={{ manifest_url | urlencode }}"
        class="btn"
    >
        Tap to install
    </a>
    {% if ipa.minimum_os_version %}
    <p class="comment">
        This app requires iOS {{ ipa.minimum_os_version }} or higher.
    </p>
    {% endif %}
</body>
</html>
'''

DEFAULT_MANIFEST = '''
<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
    <dict>
        <key>items</key>
        <array>
            <dict>
                <key>assets</key>
                <array>
                    <dict>
                        <key>kind</key>
                        <string>software-package</string>
                        <key>url</key>
                        <string>{{ ipa_url | escape }}</string>
                    </dict>
                    {% if image_url %}
                    <dict>
                        <key>kind</key>
                        <string>display-image</string>
                        <key>needs-shine</key>
                        <false/>
                        <key>url</key>
                        <string>{{ image_url | escape }}</string>
                    </dict>
                    {% endif %}
                </array>
                <key>metadata</key>
                <dict>
                    <key>bundle-identifier</key>
                    <string>{{ ipa.id }}</string>
                    <key>bundle-version</key>
                    <string>{{ ipa.short_version }}</string>
                    <key>kind</key>
                    <string>software</string>
                    <key>title</key>
                    <string>{{ ipa.display_name }}</string>
                </dict>
            </dict>
        </array>
    </dict>
</plist>
'''

def render_template(template, context):
    env = jinja2.Environment(autoescape=True)
    t = env.from_string(template)
    return t.render(**context)
